use strict;
use HTTP::Request;
use LWP::UserAgent;
use Time::Local;
use XML::Simple;

#use Data::Dumper;

package VOPL;

=head1 NAME

VOPL - Allows you to query Veracode's Results API.

=head1 SYNOPSIS

use VOPL;

my $vopl=VOPL->new($user,$passwd,$server,$port);

my @allbuildids=$vopl->getallreadybuildids();

=head1 DESCRIPTION

This library allows you to query Veracode's Results API programmatically. The
APIs are defined below.

=head1 APIs

=head2 new

This API returns an instance of a VOPL object.

=head3 Usage:

my $vopl=VOPL->new($user,$passwd,$server,$port);

=cut

sub new {
	my ($type,$user,$passwd,$server,$port)=(@_);
	my $self={	'USER'=>$user,
			'PASSWD'=>$passwd,
			'UA'=>LWP::UserAgent->new,
			'SERVER'=>$server, 'PORT'=>$port,
			'XS'=>XML::Simple->new(),
	};
	$self->{UA}->agent("VOPL");
	# The null for these credentials are magic in that I do not know
	# how HTTP::Request determines the value.
	$self->{UA}->credentials("$server:$port","null",$user,$passwd);
	#let us bless this mess.
	bless $self, $type;
}

=head2 getallreadyresults

This API returns an Array of Hashes. Each element in the array represents a
build with published results and those XML results turned into a Hash of
Hashes. Please note this can take a while to run.

=head3 Usage:

my $vopl=VOPL->new($user,$passwd,$server,$port);
my @results=$vopl->getallreadyresults();

for(@results) {
	do_work_on_this_run($_);
}

=cut

sub getallreadyresults {
	my $self=shift;
	return $self->getbuildids(
		map {	if(!/Build Id:(\d+)/) {
				die "Was unable to find Build Id in $_";
			}
			$1;
		} $self->getallreadybuildids());
}

=head2 getallreadyresultsinxml

This API returns an Array. Each element in the array represents a
build with published results and the raw XML. Please note this can take a while
to run.

=head3 Usage:

my $vopl=VOPL->new($user,$passwd,$server,$port);
my @results=$vopl->getallreadyresultsinxml();

for(@results) {
	print_to_file($_);
}

=cut

sub getallreadyresultsinxml {
	my $self=shift;
	return $self->getbuildidsinxml(
		map {	if(!/Build Id:(\d+)/) {
				die "Was unable to find Build Id in $_";
			}
			$1;
		} $self->getallreadybuildids());
}

=head2 getallreadybuildids

This API returns an Array. Each element in the array is a string that is a list
of App Ids and Build Ids.

=head3 Usage:

my $vopl=VOPL->new($user,$passwd,$server,$port);
my @results=$vopl->getallreadybuildids();

for(@results) {

	/App Id:(\d+) Build Id:(\d+)/;
	my ($appid,$buildid)=($1,$2);
	....

}

=cut

sub getallreadybuildids {
	my $self=shift;
	my $list="";
	eval { $list=$self->getlistapps(1); };
	if ($@) { die "was unable to get a list of apps:$@"; }
	#Yea this should be a map {}grep{} but I am tired so.
	my @answer;
	for (keys %{$list}) {
		if (exists $list->{$_}->{last_published_epoc}) {
			my ($aid,$bid)=split /:/;
			push @answer, "App Id:$aid\tBuild Id:$bid";
		}
	}
	return @answer;
}

# Please note that this API returns a HoAoH and
# one of the keys in 2 of the HoAoH's is status. 
sub _convertlst {
	my ($self,$list)=@_;
	my $ret={};
	for my $app(@{$list->{application}}) {
		my $appid=$app->{app_id};
		my $res=();
		my @builds=($app->{build});
		if(ref($app->{build}) eq "ARRAY") { @builds=@{$app->{build}}; }
		$res=[map {
			my %ans=(	appid=>$appid,
					buildid=>defined($_->{build_id})?
						$_->{build_id}:"",
					appname=>$app->{app_name},
				);

			if(ref($_->{analysis_unit}) eq "HASH") {
				my @res=($self->_createstatus
						($_->{analysis_unit}));
				$ans{status}=\@res;
				if(exists $res[0]->{published_date_epoc}) {
					$ans{last_published_epoc}=
						$res[0]->{published_date_epoc};
				}
			} else {
				my @res=map {$self->_createstatus($_) } 
					@{$_->{analysis_unit}};
				$ans{status}=\@res;
				my $lastest=0;
				for(@res) {
					if(!exists $_->{published_date_epoc}) {
						next;
					}
					if($lastest<$_->{published_date_epoc}) {
						$lastest=$_->{published_date_epoc};
					}
				}
				if ($lastest !=0) {
					$ans{last_published_epoc}=$lastest;
				}
			}
			\%ans;
		   } @builds];
		for(@$res) {
			$ret->{$_->{appid}.":".$_->{buildid}}=$_;
		}
	}
	return $ret;
}

sub _createstatus {
	my ($self,$unit)=@_;
	my $res={};
	$res->{status}=$unit->{status};
	$res->{type}=$unit->{analysis_type};
	if (exists $unit->{published_date}) {
		$res->{published_date}=$unit->{published_date};
		my ($year,$mon,$day,$hour,$min,$sec,$tzone1,$tzone2)=
				split /[-T:\+]/,$unit->{published_date};
		$mon--;
		#Ignoring the time zone info, I am hoping that
		#the timezone being given to us is the same as
		#the local time zone.
		$res->{published_date_epoc}=Time::Local::timelocal
			($sec,$min,$hour,$day,$mon,$year);
	}
	return $res;
}

=head2 getlistapps

This API returns a hash. This hash has a key called application which points to
an array that is a list of all of the applications that your account has and
details about each build that was submitted.

=head3 Usage:

my $vopl=VOPL->new($user,$passwd,$server,$port);
my $results=$vopl->getlistapps();

my $apps=$results->{application};

for(@$apps) {

	print Data::Dumper->Dump([\$_]);

}

=cut

sub getlistapps {
	my ($self,$arg)=(@_,0);
	my $res=$self->{XS}->XMLin($self->getlistappsinxml());
	if($arg) { 
		$res=$self->_convertlst($res);
	}
	return $res;
}

=head2 getlistappsinxml

This API returns a string that is the XML that the Veracode Results API returned.

=head3 Usage:

my $vopl=VOPL->new($user,$passwd,$server,$port);
my $xml=$vopl->getlistappsinxml();

=cut
 
sub getlistappsinxml {
	my $self=shift;
	my ($user,$passwd)=($self->{USER},$self->{PASSWD});

	my $response=$self->{UA}->request(
		HTTP::Request->new(GET=>
			'https://'.$self->{SERVER}.":".$self->{PORT}.
				'/api/getappbuilds.do'));
	if (!$response->is_success) {
		die "Bad response getting listapps: ".$response->status_line;
	}
	return $response->content;
}

=head2 getbuildids

This API takes an array of build ids for which you wish to get the results. This
returns an array where each element gives details about each flaw found with
that build id.

The order in which the results is returned will be in the same order as given.

=head3 Usage:

my $vopl=VOPL->new($user,$passwd,$server,$port);
my @results=$vopl->getallreadybuildids();

my @interestingbuildids;

for(@results) {

	/App Id:(\d+) Build Id:(\d+)/;
	my ($appid,$buildid)=($1,$2);

	if($appid != 10002) { continue; }

	push @interestingbuildids, $buildid;
}

my @res=vopl->getbuildids(@interestingbuildids);

=cut
 
sub getbuildids {
	my $self=shift;
	return (map { if($_ ne "") {$self->{XS}->XMLin($_); }} 
		$self->getbuildidsinxml(@_));
}

=head2 getbuildidsinxml

This API takes an array of build ids for which you wish to get the results. This
returns an array where each element is the xml results of that build id.

The order in which the results is returned will be in the same order as given.

=head3 Usage:

my $vopl=VOPL->new($user,$passwd,$server,$port);
my @results=$vopl->getallreadybuildids();

my @interestingbuildids;

for(@results) {

	/App Id:(\d+) Build Id:(\d+)/;
	my ($appid,$buildid)=($1,$2);

	if($appid != 10002) { continue; }

	push @interestingbuildids, $buildid;
}

my @res=vopl->getbuildidsinxml(@interestingbuildids);

=cut
 

sub getbuildidsinxml {
	my $self=shift;
	my $baseurl='https://'.$self->{SERVER}.":".$self->{PORT}.
			'/api/detailedreport.do?build_id=';
	my @ret;
	for(@_) {
		my $url=$baseurl.$_;
		my $response=$self->{UA}->request(
			HTTP::Request->new(GET=>$url));
		if (!$response->is_success) {
			if($response->code == 400) {
				die "Not allowed to get a detailed report of ".
					"build id:$_";
			}
			die "Build Id:$_ Error response in getting ".
				"detailed report.:".$response->status_line;
		} else {
			push @ret, $response->content;
		}
	}
	return @ret;
}

=head1 AUTHOR

Eric Bergeron <ebergeron@veracode.com>

=cut

"This statement is false but a true value.";
